/*30-10-2007 MRC-Epid JHZ*/

options compress=yes spool mprint;
libname epic5k '~/mrc/projects/epic5k';
libname wide '/data/genetics/GWA/6-5-7/wide';

%macro swap(data);
%do i=1 %to 2;
data a&i (keep=&snplist &alist &clist &glist &tlist);
     set &data;
     array snp &snplist;
     array s$ &slist;
     array a &alist;
     array c &clist;
     array g &glist;
     array t &tlist;
     format &alist &clist &glist &tlist 1.;
     do over snp;
        s=substr(upcase(snp),(2*&i-1),1);
        select (s);
        when ('A') a=1;
        when ('C') c=1;
        when ('G') g=1;
        when ('T') t=1;
        otherwise;
        end;
     end;
     do over snp;
        snp=s;
     end;
run;
%end;
data ab;
     set a1 a2;
run;
%macro summary(list,letter);
ods select none;
proc summary data=ab;
     id &snplist;
     var &list;
     output out=sum&letter n=;
run;
ods select all;
data sum&letter (keep=&snplist);
     set sum&letter;
     array snp &snplist;
     array allele &list;
     do over snp;
        snp=put(allele,6.);
     end;
run;
%mend summary;
%summary(&alist,a);
%summary(&clist,c);
%summary(&glist,g);
%summary(&tlist,t);
data acgt;
     set suma sumc sumg sumt;
run;
proc transpose data=acgt out=sum;
     var &snplist;
run;
proc sort data=sum (rename=(_name_=rsn));
     by rsn;
run;
data map&chr (drop=rsnn);
     set epic5k.map&chr (rename=(rsn=rsnn));
     rsn=tranwrd(rsnn,"A-","A_");
run;
proc sql;
     create table map as select * from map&chr
            where rsn in (select rsn from sum) order by rsn;
quit;
data codes (drop=location t n1 n2);
     merge sum map;
     by rsn;
     n1=input(translate(allele1,'1234','ACGT'),1.);
     n2=input(translate(allele2,'1234','ACGT'),1.);
     a1=allele1;
     a2=allele2;
     array n col1-col4;
     if n[n1]>n[n2] then do;
        t=a1;
        a1=a2;
        a2=t;
     end;
run;
%mend swap;

%macro encode(chr,data,out,start=2,ns=1,nonsnps=id,format="long",skip=no);
%let fid=%sysfunc(open(&data));
%let nvars=%sysfunc(attrn(&fid,nvars));
%let end=%eval(&nvars);
%let allsnps=%eval(&end-&start+1);
%do group=1 %to &ns;
    %if &skip=no %then %do;
    %let low=%sysfunc(ceil(&allsnps/&ns*(&group-1)+1));
    %let up=%sysfunc(ceil(&allsnps/&ns*&group));
    %let nsnps=%eval(&up-&low+1);
    %let low=%eval(&low+&start-1);
    %let up=%eval(&up+&start-1);
    %let snp1=%sysfunc(varname(&fid,&low));
    %let snpn=%sysfunc(varname(&fid,&up));
    %let snplist=&snp1.--&snpn;
    %let slist=s1-s&nsnps;
    %let alist=a1-a&nsnps;
    %let clist=c1-c&nsnps;
    %let glist=g1-g&nsnps;
    %let tlist=t1-t&nsnps;
    %swap(&data);
    data wide.codes&chr._&group;
         set codes;
         call symput(rsn,%quote(compress(a1||a2)));
    run;
    %end;
    %else %do;
    data _null_;
         set wide.codes&chr._&group;
         call symput(rsn,%quote(compress(a1||a2)));
    run;    
    %end;
%end;
%let snp1=%sysfunc(varname(&fid,&start));
%let snpn=%sysfunc(varname(&fid,&end));
%let snplist=&snp1.--&snpn;
%let rc=%sysfunc(close(&fid));
proc sql;
     drop table &out;     
quit;
%if &format="wide" %then %do;
data &out (drop=rsn a1a2 l1l2 l1 g g1 g2 wid rc i j);
     wid=open("&data");
     set &data;
     array snps{*} &snplist;
     do i=&start to &end;
        rsn=varname(wid,i);
        j=i-&start+1;
        a1a2=snps[j];
        g1=substr(a1a2,1,1);
        g2=substr(a1a2,3,1);
        l1l2=symget(rsn);
        l1=substr(l1l2,1,1);
        g=(g1=l1)+(g2=l1);
        snps[j]=put(g,6.);
     end;
     rc=close(wid);
run;
%end;
%else %if &format="long" %then %do;
data &out (keep=&nonsnps rsn g);
     lid=open("&data");
     set &data (keep=&nonsnps &snplist);
     array snps{*} &snplist;
     do i=&start to &end;
         rsn=varname(lid,i);
         j=i-&start+1;
         a1a2=snps[j];
         g1=substr(a1a2,1,1);
         g2=substr(a1a2,3,1);
         l1l2=symget(rsn);
         l1=substr(l1l2,1,1);
         l2=substr(l1l2,2,1);
         g=(g1=l1)+(g2=l1);
         output;
     end;
     rc=close(lid);
run;
%end;
%mend encode;

/*
data test;
id=_n_;
input SNP_A_1787722$ SNP_A_1791650$ SNP_A_1795980$ SNP_A_1812684$ SNP_A_1814708$;
datalines;
 G/G        T/T        C/T        G/G        T/T
 G/G        T/T        T/T        G/G        C/T
 G/G        T/T        T/T        G/G        T/T
 G/G        C/T        T/T        G/G        C/T
 G/G        C/T        T/T        G/G        T/T
 G/G        T/T        T/T        G/G        C/T
 G/G        C/T        T/T        G/G        T/T
 G/G        T/T        T/T        G/G        C/T
 G/G        T/T        T/T        G/G        T/T
 G/G        C/T        T/T        G/G        C/C
;
proc contents data=test varnum;
run;
libname x '.';
%encode(22,test,x.wide,ns=3,format="wide");
%encode(22,test,x.long,ns=3,format="long");
*/
