/*14-2-2008 MRC-Epid JHZ, SL*/

%macro fisher(data,n,vlist=ppool k);
data fisher (keep=&vlist);
     set &data;
     array p{*} p1-p&n;
     logp=0;
     k=0;
     do i=1 to &n;
        if p[i]^=. then do;
           if p[i]=0 then t=constant('LOGSMALL');
           else t=log(p[i]);
           logp=logp+t;
           k=k+1;
        end;
     end;
     if k>1 then ppool=1-probchi(-2*logp,2*k);
     eps=constant('SMALL');
     if ppool<eps then ppool=eps;
     label ppool='Combined p value(s)' k='No of tests used' eps='smallest double-precision number';
run;
%mend fisher;

%macro metareg(data,n,vlist=beta_f se_f z_f p_f beta_r se_r p_r p_heter isq k eps);
data metareg (keep=&vlist);
     set &data;
     eps=constant('SMALL');
     array b{*} b1-b&n;
     array se{*} se1-se&n;
     array w{*} w1-w&n;
     array ok{*} ok1-ok&n;
     k=0;
     bw=0;
     sw=0;
     sww=0;
     do i=1 to &n;
        ok[i]=0;
        if b[i]+se[i]>. then do;
           ok[i]=1;
           t=max(se[i],eps);
           k=k+1;
           w[i]=1/t**2;
           bw=bw+b[i]*w[i];
           sw=sw+w[i];
           ww=w[i]*w[i];
           sww=sww+ww;
        end;
     end;
     if k>1 then do;
       beta_f=bw/sw;
       se_f=sqrt(1/sw);
       z_f=beta_f/se_f;
       p_f=2*probnorm(-abs(z_f));
     end;  
     qw=0;
     do i=1 to &n;
        if ok[i]=1 then qw=qw+w[i]*(b[i]-beta_f)**2;
     end;
     if k>1 then do;
       p_heter=1-probchi(qw,k-1);
       dl=max(0,(qw-(k-1))/(sw-sww/sw));
       bwr=0;
       swc=0;
       array wc{*} wc1-wc&n;
       do i=1 to &n;
          if ok[i]=1 then do;
             wc[i]=1/(1/w[i]+dl);
             bwr=bwr+b[i]*wc[i];
             swc=swc+wc[i];
          end;
       end;
       beta_r=bwr/swc;
       se_r=sqrt(1/swc);
       z_r=beta_r/se_r;
       p_r=2*probnorm(-abs(z_r));
       isq=(qw-k+1)/qw;
       if .<isq<0 then isq=0;
     end;
     label p_f='P value (fixed effects model)'  p_r='P value (random effects model)'
           beta_f='regression coefficient'      beta_r='regression coefficient'
           se_f='standard error'                se_r='standard error'
           z_f='z value'                        z_r='z value'
           p_heter='heterogeneity test p value'
           isq='I^2'
           k='No of tests used'
           eps='smallest double-precision number';
run;
%mend metareg;

/*
data abc;
     chromosome=1;rsn='abcd';startpos=1234;
     b1=1;se1=2;p1=0.1;
     b2=2;se2=6;p2=0;
     b3=3;se3=8;p3=0.5;
run;
options mprint;
%fisher(abc,3,vlist=rsn chromosome startpos ppool k);
proc print label;
     var rsn chromosome startpos ppool k; 
     where 0.0001>ppool>.;
run;
%metareg(abc,3);
proc print label;
run;
*/

%macro sketch;
/*b1-bn are betas from regression model*/
 k=n; /*n is the number of studies to be combined*/
 w1=1/se1**2; /*weight for study 1*/
 w2=1/se2**2; /*weight for study 2*/
 ...............
 wn=1/sen**2; /*weight for study n*/
 w=w1+w2+...+wn; /*total weight*/
 beta_f=(b1*w1+b2*w2+...+bn*wn)/w; /*combined beta from fixed effectmodel*/
 se_f=sqrt(1/w);/*combined se from fixed effect model*/
 z_f=beta_f/se_f;/*combined z-score from fixed effect model*/
 p_f=2*(1-probnorm(abs(z_f)));/*combined p value from fixed effect model*/
 qw=w1*(b1-beta_f)**2+w2*(b2-beta_f)**2+...+wn*(bn-beta_f)**2;
/*statistics for test of heterogeneity*/
 dl=max(0,(qw-(k-1))/(w-(w1**2+w2**2+...+wn**2)/w));
 w1_c=1/(1/w1+dl); /*corrected weight for study 1*/
 w2_c=1/(1/w2+dl); /*corrected weight for study 2*/
 ............
 wn_c=1/(1/wn+dl); /*corrected weight for study n*/
 w_c=w1_c+w2_c+...+wn_c; /*total corrected weight*/
 beta_r=(b1*w1_c+b2*w2_c+...+bn*wn_c)/w_c;/*combined beta from random effect model*/
 se_r=sqrt(1/w_c);/*combined se from random effect model*/
 z_r=beta_r/se_r;/*combined z-score from random effect model*/
 p_r=2*(1-probnorm(abs(z_r)));/*combined p value from random effect model*/
 pheter=1-probchi(qw,k-1); /*p value for heterogeneity test*/
%mend sketch;
