/*23-10-2007 MRC-Epid JHZ*/

%macro reshape (file,nobs=3552);
%let all=%eval(&nobs*3);
data xyz (drop=i j);
     infile &file lrecl=80000;
     format id1 id2 $15.;
     format a1 a2 $1.;
     input id1$ id2$ pos a1$ a2$ u1-u&all;
     array u u1-u&all;
     array x x1-x&nobs;
     array y y1-y&nobs;
     array z z1-z&nobs;
     do while (i le &nobs);
        i=1;
        x[i]=u[3*i-2];
        y[i]=u[3*i-1];
        z[i]=u[3*i];
     end;
run;
proc sql;
     create table x as select * from xyz (keep=id1 id2 pos x1-x&nobs) order by pos, id2;
     create table y as select * from xyz (keep=id1 id2 pos y1-y&nobs) order by pos, id2;
     create table z as select * from xyz (keep=id1 id2 pos z1-z&nobs) order by pos, id2;
quit;
proc transpose data=x out=xx;
     var x1-x&nobs;
     by pos id2;
run;
proc transpose data=y out=yy;
     var y1-y&nobs;
     by pos id2;
run;
proc transpose data=z out=zz;
     var z1-z&nobs;
     by pos id2;
run;
data xxyyzz;
     merge xx (rename=(col1=g0)) yy (rename=(col1=g1)) zz (rename=(col1=g2));
     by pos id2;
     if first.pos then iid=1;
     else iid+1;
run;
proc transpose data=xxyyzz (drop=_NAME_) out=xyz (rename=(col1=g) drop=_NAME_);
     var g0-g2;
     by pos id2 iid;
run;
proc sort data=xyz;
     by iid id2;
run;
%mend reshape;

%macro snptest(i);
%do j=1 %to 30;
%reshape("chr&i-&j..out");
data long;
     merge trait xyz;
     by iid;
run;
proc sort data=long;
     by pos id2 iid;
run;
data scratch.chr&i._&j;
     set long;
     by pos id2 iid;
     format add 1.;
     if first.iid then add=0;
     else add+1;
run;
ods select none;
proc surveylogistic data=scratch.chr&i._&j;
     ods output parameterestimates=obesity.logit&i._&j (where=(variable^="Intercept"));
     by pos id2;
     cluster iid;
     model case (descending)=add / expb;
     weight g;
run;
proc surveyreg data=scratch.chr&i._&j (where=(cohort="1"));
     ods output parameterestimates=bmi.reg&i._&j (where=(parameter^="Intercept"));
     by pos id2;
     cluster iid;
     model bmi=add / clparm;
     weight g;
run;
proc surveyreg data=scratch.chr&i._&j (where=(cohort="1"));
     ods output parameterestimates=zbmi.reg&i._&j (where=(parameter^="Intercept"));
     by pos id2;
     cluster iid;
     model zbmi=add / clparm;
     weight g;
run;
proc surveyreg data=scratch.chr&i._&j;
     ods output parameterestimates=zheight.reg&i._&j (where=(parameter^="Intercept"));
     by pos id2;
     cluster iid;
     model zheight=add / clparm;
     weight g;
run;
proc surveyreg data=scratch.chr&i._&j;
     ods output parameterestimates=hdl.reg&i._&j (where=(parameter^="Intercept"));
     by pos id2;
     cluster iid;
     model hdl=add / clparm;
     weight g;
run;
proc surveyreg data=scratch.chr&i._&j;
     ods output parameterestimates=hba1c.reg&i._&j (where=(parameter^="Intercept"));
     by pos id2;
     cluster iid;
     model hba1c=add / clparm;
     weight g;
run;
proc sql;
     drop table long;
     drop table scratch.chr&i._&j;
quit;
ods select all;
%end;
%mend snptest;

title;
options ps=max nocenter nodate nonumber nonotes;
libname epic5k '/data/genetics/GWA/sas';
libname scratch '/data/genetics/scratch/jhz22';
libname obesity 'obesity';
libname bmi 'bmi';
libname zbmi 'zbmi';
libname zheight 'zheight';
libname hdl 'hdl';
libname hba1c 'hba1c';
proc format;
     value agefmt low-<50='1' 50-<60='2' 60-<70='3' 70-high='4';
run;
data trait (drop=id age sex height);
     set epic5k.trait;
     iid=_n_;
run;

%snptest(1);
%snptest(2);
%snptest(3);
%snptest(4);
%snptest(5);
%snptest(6);
%snptest(7);
%snptest(8);
%snptest(9);
%snptest(10);
%snptest(11);
%snptest(12);
%snptest(13);
%snptest(14);
%snptest(15);
%snptest(16);
%snptest(17);
%snptest(18);
%snptest(19);
%snptest(20);
%snptest(21);
%snptest(22);

/*
proc transpose data=epic5k.trait out=ttrait (drop=_name_ _label_);
     id id;
     var bmi;
run;
proc transpose data=obesity.long&j out=scratch.long&j (drop=_name_);
     id id;
     by rsn;
     var a1a2;
run;
data xyz (drop=g0-g2);
     set xxyyzz (rename=(_NAME_=id));
     by pos id2;
     g=g0;output;
     g=g1;output;
     g=g2;output;
run;
proc export data=scratch.chr&i._&j outfile="chr&i._&j.csv" replace dbms=csv;
run;
*/
